#include <QMainWindow>
#include <QMdiArea>
#include <QDebug>
#include <QPushButton>
#include <QStyle>
#include <QApplication>
#include <QMdiSubWindow>
#include <QVBoxLayout>

class WidgetZoomCrash : public QMainWindow
{
    Q_OBJECT

public:
    WidgetZoomCrash(QWidget *parent = 0);

public slots:
    void button1Clicked() { resizeSubs(QSize(400, 300)); }
    void button2Clicked() { resizeSubs(QSize(12000, 4000)); }
    void button3Clicked() { resizeSubs(m_mdiArea->subWindowList().first()->size() * 2); }
    void button4Clicked() { resizeSubs(m_mdiArea->subWindowList().first()->size() * 4); }

private:
    void resizeSubs(const QSize &);

    QMdiArea *m_mdiArea;
};

WidgetZoomCrash::WidgetZoomCrash(QWidget *parent) : QMainWindow(parent)
{
  QPushButton *button1 = new QPushButton("ORIG");
  connect(button1, SIGNAL(clicked()), this, SLOT(button1Clicked()));

  QPushButton *button2 = new QPushButton("12000 x 4000");
  connect(button2, SIGNAL(clicked()), this, SLOT(button2Clicked()));

  QPushButton *button3 = new QPushButton("200%");
  connect(button3, SIGNAL(clicked()), this, SLOT(button3Clicked()));

  QPushButton *button4 = new QPushButton("400%");
  connect(button4, SIGNAL(clicked()), this, SLOT(button4Clicked()));

  m_mdiArea = new QMdiArea(this);
  m_mdiArea->setOption(QMdiArea::DontMaximizeSubWindowOnActivation);
  m_mdiArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
  m_mdiArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
  m_mdiArea->resize(400, 300);

  Qt::WindowFlags flags = Qt::CustomizeWindowHint | Qt::WindowSystemMenuHint |
                          Qt::WindowTitleHint | Qt::WindowMinimizeButtonHint;

  QWidget *widget = new QWidget;
  widget->resize(400, 300);

  QMdiSubWindow *subwindow = m_mdiArea->addSubWindow(widget, flags);
  subwindow->resize(400, 300);
  subwindow->setWindowTitle("MySubWindow");
  subwindow->show();

  QVBoxLayout *vlayout = new QVBoxLayout;
  vlayout->addWidget(button1);
  vlayout->addWidget(button2);
  vlayout->addWidget(button3);
  vlayout->addWidget(button4);
  vlayout->addWidget(m_mdiArea);

  QWidget *w = new QWidget;
  w->setLayout(vlayout);

  setCentralWidget(w);

  button1->show();

  setWindowTitle(QT_VERSION_STR + QLatin1Char(' ') + QApplication::style()->objectName());
  setFixedSize(600, 600);
}

void WidgetZoomCrash::resizeSubs(const QSize &s)
{
    qDebug() << __FUNCTION__ << s;
    foreach(QMdiSubWindow *subWindow,  m_mdiArea->subWindowList())
        subWindow->resize(s);
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    WidgetZoomCrash test;
    qDebug() << QT_VERSION_STR;
    test.show();
    return app.exec();
}

#include "main.moc"
